PROGRAM example1
USE tensolve
IMPLICIT NONE

!     TENSOLVE finds roots of systems of n nonlinear equations in n unknowns,
!     or minimizers of the sum of squares of m > n nonlinear functions in
!     n unknowns, using tensor methods.

!     This example illustrates the use of TENSOLVE to solve a nonlinear
!     equation problem defined by the subroutine frosen (included below).

INTEGER            :: m, n, msg, termcd
INTEGER, PARAMETER :: maxm = 100, maxn = 30, maxp = 6
REAL (dp)          :: x0(maxn), xp(maxn), fp(maxm), gp(maxn)

INTERFACE
  SUBROUTINE frosen ( x, f, m, n )
    USE tensolve, ONLY: dp
    IMPLICIT NONE
    REAL (dp), INTENT(IN)             :: x(:)
    REAL (dp), INTENT(OUT)            :: f(:)
    INTEGER, INTENT(IN)               :: m
    INTEGER, INTENT(IN)               :: n
  END SUBROUTINE frosen
END INTERFACE

!     Set dimensions of the problem.

m      = 2
n      = 2

!     Set values for the initial point.

x0(1)  = -1.20D0
x0(2)  = 1.0D0
msg    = 0

!     Call TENSOLVE.

CALL tsnesi(maxm, maxn, maxp, x0, m, n, frosen, msg, xp, fp, gp, termcd )

WRITE(*, '(a, i5)')     ' Termination code = ', termcd
WRITE(*, '(a, 2g14.6)') ' Solution: ', xp(1:2)
WRITE(*, '(a, 2g14.6)') ' Function values at solution = ', fp(1:2)
WRITE(*, '(a, 2g14.6)') ' Gradients at solution = ', gp(1:2)

!     end of example1 main program.
STOP
END PROGRAM example1


SUBROUTINE frosen ( x, f, m, n )
USE tensolve, ONLY: dp
IMPLICIT NONE
REAL (dp), INTENT(IN)   :: x(:)
REAL (dp), INTENT(OUT)  :: f(:)
INTEGER, INTENT(IN)     :: m
INTEGER, INTENT(IN)     :: n

!     frosen defines function values for the Rosenbrock function.

f(1) = 10.0D0 * (x(n) - x(1)**2)
f(m) = 1.0D0 - x(1)

!     end of frosen.
RETURN
END SUBROUTINE frosen
